'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; /*
                                                                                                                                                                                                                                                                              * Copyright 2018 ARDUINO SA (http://www.arduino.cc/)
                                                                                                                                                                                                                                                                              * This file is part of arduino-iot-js.
                                                                                                                                                                                                                                                                              * Copyright (c) 2018
                                                                                                                                                                                                                                                                              * Authors: Fabrizio Mirabito
                                                                                                                                                                                                                                                                              *
                                                                                                                                                                                                                                                                              * This software is released under:
                                                                                                                                                                                                                                                                              * The GNU General Public License, which covers the main part of
                                                                                                                                                                                                                                                                              * arduino-iot-js
                                                                                                                                                                                                                                                                              * The terms of this license can be found at:
                                                                                                                                                                                                                                                                              * https://www.gnu.org/licenses/gpl-3.0.en.html
                                                                                                                                                                                                                                                                              *
                                                                                                                                                                                                                                                                              * You can be released from the requirements of the above licenses by purchasing
                                                                                                                                                                                                                                                                              * a commercial license. Buying such a license is mandatory if you want to modify or
                                                                                                                                                                                                                                                                              * otherwise use the software for commercial activities involving the Arduino
                                                                                                                                                                                                                                                                              * software without disclosing the source code of your own applications. To purchase
                                                                                                                                                                                                                                                                              * a commercial license, send an email to license@arduino.cc.
                                                                                                                                                                                                                                                                              *
                                                                                                                                                                                                                                                                              */

var _pahoClient = require('paho-client');

var _pahoClient2 = _interopRequireDefault(_pahoClient);

var _cborJs = require('cbor-js');

var _cborJs2 = _interopRequireDefault(_cborJs);

var _ArduinoCloudError = require('./ArduinoCloudError');

var _ArduinoCloudError2 = _interopRequireDefault(_ArduinoCloudError);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var connections = {};
var propertyCallback = {};
var arduinoCloudPort = 8443;
var arduinoCloudHost = 'wss.iot.arduino.cc';
var arduinoAuthURL = 'https://auth.arduino.cc';

var getUserId = function getUserId(apiUrl, token) {
  return fetch(apiUrl, {
    method: 'get',
    headers: new Headers({
      Authorization: 'Bearer ' + token,
      'Content-Type': 'application/json'
    })
  }).then(function (res) {
    return res.json();
  });
};

// Connect establishes a connection with mqtt, using token as the password, and returns a promise
// of a Symbol identifying the mqtt client
var connect = function connect(options) {
  return new Promise(function (resolve, reject) {
    var ssl = false;
    if (options.ssl !== false) {
      ssl = true;
    }
    var opts = {
      host: options.host || arduinoCloudHost,
      port: options.port || arduinoCloudPort,
      apiUrl: options.apiUrl || arduinoAuthURL,
      ssl: ssl,
      token: options.token,
      onDisconnect: options.onDisconnect
    };

    if (!opts.host) {
      return reject(new Error('connection failed: you need to provide a valid host (broker)'));
    }

    if (!opts.token) {
      return reject(new Error('connection failed: you need to provide a valid token'));
    }

    if (!opts.apiUrl) {
      return reject(new Error('no apiUrl parameter is provided'));
    }

    return getUserId(opts.apiUrl + '/v1/users/byID/me', options.token).then(function (res) {
      var clientID = res.id + ':' + new Date().getTime();
      var client = new _pahoClient2.default.Client(opts.host, opts.port, clientID);
      client.topics = {};
      client.properties = {};

      client.onMessageArrived = function (msg) {
        if (msg.topic.indexOf('/s/o') > -1) {
          client.topics[msg.topic].forEach(function (cb) {
            cb(msg.payloadString);
          });
        } else {
          var buf = new ArrayBuffer(msg.payloadBytes.length);
          var bufView = new Uint8Array(buf);
          for (var i = 0, strLen = msg.payloadBytes.length; i < strLen; i += 1) {
            bufView[i] = msg.payloadBytes[i];
          }

          var propertyValue = _cborJs2.default.decode(buf);
          propertyValue.forEach(function (p) {
            if (propertyCallback[msg.topic][p.n]) {
              var value = null;

              if (!(p.v === undefined)) {
                value = p.v;
              } else if (!(p.vs === undefined)) {
                value = p.vs;
              } else if (!(p.vb === undefined)) {
                value = p.vb;
              }

              propertyCallback[msg.topic][p.n](value);
            }
          });
        }
      };

      if (typeof onDisconnect === 'function') {
        client.onConnectionLost = opts.onDisconnect;
      }

      var connectionOpts = {
        useSSL: opts.ssl,
        timeout: 30,
        mqttVersion: 4,
        userName: res.id,
        // password: token,
        mqttVersionExplicit: true,
        // If reconnect is set to true, in the event that the connection is lost, the client will
        // attempt to reconnect to the server. It will initially wait 1 second before it attempts
        // to reconnect, for every failed reconnect attempt, the delay will double until it is at
        // 2 minutes at which point the delay will stay at 2 minutes.
        reconnect: true,
        keepAliveInterval: 30,
        onSuccess: function onSuccess() {
          var id = Symbol(clientID);
          connections[id] = client;
          return resolve(id);
        },
        onFailure: function onFailure(_ref) {
          var errorCode = _ref.errorCode,
              errorMessage = _ref.errorMessage;
          return reject(new _ArduinoCloudError2.default(errorCode, errorMessage));
        }
      };

      connectionOpts.password = opts.token;

      client.connect(connectionOpts);
    }, function (error) {
      throw new Error(error);
    });
  });
};

var disconnect = function disconnect(id) {
  return new Promise(function (resolve, reject) {
    var client = connections[id];
    if (!client) {
      return reject(new Error('disconnection failed: client not found'));
    }

    client.disconnect();
    return resolve();
  });
};

var subscribe = function subscribe(id, topic, cb) {
  return new Promise(function (resolve, reject) {
    var client = connections[id];
    if (!client) {
      return reject(new Error('disconnection failed: client not found'));
    }

    return client.subscribe(topic, {
      onSuccess: function onSuccess() {
        if (!client.topics[topic]) {
          client.topics[topic] = [];
        }
        client.topics[topic].push(cb);
        return resolve(topic);
      },
      onFailure: function onFailure() {
        return reject();
      }
    });
  });
};

var unsubscribe = function unsubscribe(id, topic) {
  return new Promise(function (resolve, reject) {
    var client = connections[id];
    if (!client) {
      return reject(new Error('disconnection failed: client not found'));
    }

    return client.unsubscribe(topic, {
      onSuccess: function onSuccess() {
        return resolve(topic);
      },
      onFailure: function onFailure() {
        return reject();
      }
    });
  });
};

var arrayBufferToBase64 = function arrayBufferToBase64(buffer) {
  var binary = '';
  var bytes = new Uint8Array(buffer);
  var len = bytes.byteLength;
  for (var i = 0; i < len; i += 1) {
    binary += String.fromCharCode(bytes[i]);
  }
  return window.btoa(binary);
};

var sendMessage = function sendMessage(id, topic, message) {
  return new Promise(function (resolve, reject) {
    var client = connections[id];
    if (!client) {
      return reject(new Error('disconnection failed: client not found'));
    }

    client.publish(topic, message, 1, false);
    return resolve();
  });
};

var openCloudMonitor = function openCloudMonitor(id, deviceId, cb) {
  var cloudMonitorOutputTopic = '/a/d/' + deviceId + '/s/o';
  return subscribe(id, cloudMonitorOutputTopic, cb);
};

var writeCloudMonitor = function writeCloudMonitor(id, deviceId, message) {
  var cloudMonitorInputTopic = '/a/d/' + deviceId + '/s/i';
  return sendMessage(id, cloudMonitorInputTopic, message);
};

var closeCloudMonitor = function closeCloudMonitor(id, deviceId) {
  var cloudMonitorOutputTopic = '/a/d/' + deviceId + '/s/o';
  return unsubscribe(id, cloudMonitorOutputTopic);
};

var sendProperty = function sendProperty(connectionId, deviceId, name, value, timestamp) {
  var propertyInputTopic = '/a/d/' + deviceId + '/e/i';

  if (timestamp && !Number.isInteger(timestamp)) {
    throw new Error('Timestamp must be Integer');
  }

  if (name === undefined || typeof name !== 'string') {
    throw new Error('Name must be a valid string');
  }

  var cborValue = {
    bt: timestamp || new Date().getTime(),
    n: name
  };

  switch (typeof value === 'undefined' ? 'undefined' : _typeof(value)) {
    case 'string':
      cborValue.vs = value;
      break;
    case 'number':
      cborValue.v = value;
      break;
    case 'boolean':
      cborValue.vb = value;
      break;
    default:
      break;
  }

  return sendMessage(connectionId, propertyInputTopic, _cborJs2.default.encode([cborValue]));
};

var getSenml = function getSenml(deviceId, name, value, timestamp) {
  if (timestamp && !Number.isInteger(timestamp)) {
    throw new Error('Timestamp must be Integer');
  }

  if (name === undefined || typeof name !== 'string') {
    throw new Error('Name must be a valid string');
  }

  var senMl = {
    bt: timestamp || new Date().getTime(),
    n: name
  };

  if (deviceId) {
    senMl.bn = 'urn:uuid:' + deviceId;
  }

  switch (typeof value === 'undefined' ? 'undefined' : _typeof(value)) {
    case 'string':
      senMl.vs = value;
      break;
    case 'number':
      senMl.v = value;
      break;
    case 'boolean':
      senMl.vb = value;
      break;
    default:
      break;
  }
  return senMl;
};

var getCborValue = function getCborValue(senMl) {
  var cborEncoded = _cborJs2.default.encode(senMl);
  return arrayBufferToBase64(cborEncoded);
};

var sendPropertyAsDevice = function sendPropertyAsDevice(connectionId, deviceId, name, value, timestamp) {
  var propertyInputTopic = '/a/d/' + deviceId + '/e/o';

  if (timestamp && !Number.isInteger(timestamp)) {
    throw new Error('Timestamp must be Integer');
  }

  if (name === undefined || typeof name !== 'string') {
    throw new Error('Name must be a valid string');
  }

  var senMlValue = getSenml(deviceId, name, value, timestamp);
  return sendMessage(connectionId, propertyInputTopic, _cborJs2.default.encode([senMlValue]));
};

var onPropertyValue = function onPropertyValue(connectionId, deviceId, name, cb) {
  if (!name) {
    throw new Error('Invalid property name');
  }
  if (typeof cb !== 'function') {
    throw new Error('Invalid callback');
  }
  var propOutputTopic = '/a/d/' + deviceId + '/e/o';

  if (!propertyCallback[propOutputTopic]) {
    propertyCallback[propOutputTopic] = {};
    propertyCallback[propOutputTopic][name] = cb;
    subscribe(connectionId, propOutputTopic, cb);
  } else if (propertyCallback[propOutputTopic] && !propertyCallback[propOutputTopic][name]) {
    propertyCallback[propOutputTopic][name] = cb;
  }
  return Promise.resolve(propOutputTopic);
};

exports.default = {
  connect: connect,
  disconnect: disconnect,
  subscribe: subscribe,
  unsubscribe: unsubscribe,
  sendMessage: sendMessage,
  openCloudMonitor: openCloudMonitor,
  writeCloudMonitor: writeCloudMonitor,
  closeCloudMonitor: closeCloudMonitor,
  sendProperty: sendProperty,
  sendPropertyAsDevice: sendPropertyAsDevice,
  onPropertyValue: onPropertyValue,
  getCborValue: getCborValue,
  getSenml: getSenml,
  ArduinoCloudError: _ArduinoCloudError2.default
};